package com.gic.auth.service.outer.impl;

import java.util.Arrays;
import java.util.Collections;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

import com.alibaba.fastjson.JSON;
import com.gic.auth.constant.ResourceHoldEnum;
import com.gic.auth.dto.*;
import com.gic.auth.entity.TabSysUser;
import com.gic.auth.service.UserService;
import com.gic.enterprise.response.EnterpriseServiceResponse;
import com.gic.store.service.StoreWidgetApiService;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang.StringUtils;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.gic.api.base.commons.Page;
import com.gic.api.base.commons.ServiceResponse;
import com.gic.application.center.api.service.AuthorizeService;
import com.gic.auth.entity.TabSysResourceGroup;
import com.gic.auth.service.ResourceGroupApiService;
import com.gic.auth.service.ResourceGroupService;
import com.gic.auth.service.UserResourceService;
import com.gic.commons.util.EntityUtil;
import com.gic.commons.util.PageHelperUtils;
import com.gic.enterprise.error.ErrorCode;

/**
 * 资源组
 * @ClassName:
 * @Description:  
 * @author guojuxing 
 * @date 2021/1/8 4:38 PM 
 */
@Service("resourceGroupApiService")
public class ResourceGroupApiServiceImpl implements ResourceGroupApiService{
    private static Logger logger = LogManager.getLogger(ResourceGroupApiServiceImpl.class);
    @Autowired
    private ResourceGroupService resourceGroupService;
    @Autowired
    private AuthorizeService authorizeService;
    @Autowired
    private UserResourceService userResourceService;
    @Autowired
    private UserService userService;
    @Autowired
    private StoreWidgetApiService storeWidgetApiService;

    @Override
    public ServiceResponse<Integer> saveResourceGroup(ResourceGroupDTO resourceGroup) {
        ServiceResponse validParamResponse = validParam(resourceGroup);
        if (!validParamResponse.isSuccess()) {
            return ServiceResponse.failure(validParamResponse.getCode(), validParamResponse.getMessage());
        }
        Integer resourceGroupId = resourceGroupService.saveResourceGroup(resourceGroup);
        return ServiceResponse.success(resourceGroupId);
    }

    @Override
    public ServiceResponse<String> editResourceGroup(ResourceGroupDTO resourceGroup) {
        ServiceResponse validParamResponse = validParam(resourceGroup);
        if (!validParamResponse.isSuccess()) {
            return ServiceResponse.failure(validParamResponse.getCode(), validParamResponse.getMessage());
        }
        resourceGroupService.editResourceGroup(resourceGroup);
        return ServiceResponse.success();
    }

    @Override
    public ServiceResponse<Page<ResourceGroupDTO>> pageResourceGroup(String name, Integer enterpriseId,
                                                                     Integer pageNum, Integer pageSize) {
        com.github.pagehelper.Page<TabSysResourceGroup> page = resourceGroupService.pageResourceGroup(name, enterpriseId, pageNum, pageSize);
        Page<ResourceGroupDTO> resultPage = PageHelperUtils.changePageHelperToCurrentPage(page, ResourceGroupDTO.class);
        List<ResourceGroupDTO> resourceList = resultPage.getResult();
        if (CollectionUtils.isNotEmpty(resourceList)) {
            //查询授权管理员数量
            List<Integer> resourceIds = resourceList.stream().map(ResourceGroupDTO::getResourceGroupId).collect(Collectors.toList());
            Map<Integer, Integer> map = userResourceService.countByResourceIds(enterpriseId, resourceIds);
            for (ResourceGroupDTO resource : resourceList) {
                Integer count = map.get(resource.getResourceGroupId());
                resource.setUserResourceCount(count == null ? 0 : count);
            }
        }
        return ServiceResponse.success(resultPage);
    }

    @Override
    public ServiceResponse<ResourceGroupDTO> getResourceGroup(Integer resourceGroupId) {
        return ServiceResponse.success(EntityUtil.changeEntityNew(ResourceGroupDTO.class,
                resourceGroupService.getResourceGroup(resourceGroupId)));
    }

    @Override
    public ServiceResponse<String> deleteResourceGroup(Integer resourceGroupId) {
        //验证是否有管理员使用中
        TabSysResourceGroup record = resourceGroupService.getResourceGroup(resourceGroupId);
        if (record == null) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "资源组不存在");
        }
        boolean exist = this.userResourceService.existUserResource(record.getEnterpriseId(), record.getResourceGroupId());
        if (exist) {
            return ServiceResponse.failure(ErrorCode.OPERATION_FAILED.getCode(), "该资源组正在使用中，不能删除");
        }
        Integer line = resourceGroupService.deleteResourceGroup(resourceGroupId);
        if (line > 0) {
            // 同步好办
            authorizeService.delResource(resourceGroupId.longValue());
        }
        return ServiceResponse.success(record.getResourceGroupName());
    }

    @Override
    public ServiceResponse<StoreResourceDTO> getStoreResourceByUserId(Integer userId) {
        TabSysUser user = userService.getUserById(userId);
        if (user.getSuperAdmin() == 1) {
            // 是超管就返回所有
            StoreResourceDTO superAdminResource = new StoreResourceDTO();
            superAdminResource.setType(ResourceHoldEnum.ALL.getCode());
            return EnterpriseServiceResponse.success(superAdminResource);
        }

        List<UserResourceDTO> userResourceList = userResourceService.listUserResourceByUserId(userId);
        StoreResourceDTO defaultResource = new StoreResourceDTO();
        defaultResource.setType(ResourceHoldEnum.DART.getCode());
        defaultResource.setStoreIdList(Collections.emptyList());
        if (CollectionUtils.isEmpty(userResourceList)) {
            return EnterpriseServiceResponse.success(defaultResource);
        }
        //资源组改成单选
        Integer resourceGroupId = userResourceList.get(0).getResourceId();
        TabSysResourceGroup resourceGroup = resourceGroupService.getResourceGroup(resourceGroupId);
        Long storeResource = resourceGroup.getStoreResource();
        if (storeResource == null) {
            return EnterpriseServiceResponse.success(defaultResource);
        }
        List<Integer> result = storeWidgetApiService.listStoreIdByStoreWidgetId(user.getEnterpriseId(), storeResource.intValue()).getResult();
        if (CollectionUtils.isNotEmpty(result)) {
            defaultResource.setStoreIdList(result);
        }
        return ServiceResponse.success(defaultResource);
    }

    @Override
    public ServiceResponse<CustomUserAreaDTO> getCustomUserByUserId(Integer userId) {
        TabSysUser user = userService.getUserById(userId);
        if (user.getSuperAdmin() == 1) {
            // 是超管就返回所有
            return EnterpriseServiceResponse.success(new CustomUserAreaDTO().setType(ResourceHoldEnum.ALL.getCode()));
        }
        List<UserResourceDTO> resourceDTOList = this.userResourceService.listUserResourceByUserId(userId);
        if (CollectionUtils.isEmpty(resourceDTOList)) {
            return EnterpriseServiceResponse.failure(ErrorCode.NOTEXISTS);
        }
        //资源组改成单选
        Integer resourceGroupId = resourceDTOList.get(0).getResourceId();
        TabSysResourceGroup resourceGroup = resourceGroupService.getResourceGroup(resourceGroupId);

        List<ResourceGroupCustomUserDTO> customUserList = JSON.parseArray(resourceGroup.getCustomUserData(), ResourceGroupCustomUserDTO.class);

        CustomUserAreaDTO result = new CustomUserAreaDTO().setType(ResourceHoldEnum.DART.getCode())
                .setCustomUser(resourceGroup.getCustomUser())
                .setWeimobOrderArea(resourceGroup.getWeimobOrderArea())
                .setStoreOrderArea(resourceGroup.getStoreOrderArea())
                .setCustomUserList(customUserList);
        if (StringUtils.isNotBlank(resourceGroup.getSmsId())) {
            result.setSmsId(Arrays.stream(resourceGroup.getSmsId()
                    .split(","))
                    .mapToLong(e -> Long.valueOf(e))
                    .boxed()
                    .collect(Collectors.toList()));
        }
        if (StringUtils.isNotBlank(resourceGroup.getWeimobStoreId())) {
            result.setWeimobStoreId(Arrays.stream(resourceGroup.getWeimobStoreId().split(","))
                    .mapToLong(e -> Long.valueOf(e))
                    .boxed()
                    .collect(Collectors.toList()));
        }
        return ServiceResponse.success(result);
    }

    /**
     * 参数验证
     * @param resourceGroup
     * @return
     */
    private ServiceResponse<Void> validParam(ResourceGroupDTO resourceGroup) {
        Integer resourceGroupId = resourceGroup.getResourceGroupId();
        if (resourceGroupId != null) {
            TabSysResourceGroup record = resourceGroupService.getResourceGroup(resourceGroupId);
            if (record == null) {
                return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "资源组不存在");
            }
        }
        //param valid
        if (StringUtils.isBlank(resourceGroup.getResourceGroupName())) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "资源组名称不能为空");
        }
        Integer enterpriseId =  resourceGroup.getEnterpriseId();
        if (enterpriseId == null) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "商户ID不能为空");
        }
        if (resourceGroupService.isRepeatResourceGroupName(resourceGroupId, resourceGroup.getResourceGroupName(), enterpriseId)) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "资源组名称重复");
        }
        Integer customUser = resourceGroup.getCustomUser();
        if (customUser == null) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "用户域授予标志不能为空");
        }
        //用户域是否授予
        boolean hasAuthCustomUser = customUser == 1;
        String customUserData = resourceGroup.getCustomUserData();
        if (hasAuthCustomUser) {
            if (StringUtils.isBlank(customUserData)) {
                return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "用户域授予资源为空，请选择不授予");
            }

            try {
                List<ResourceGroupCustomUserDTO> customUserList = JSON.parseArray(customUserData, ResourceGroupCustomUserDTO.class);
            } catch (Exception e) {
                return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "用户域数据格式有误");
            }

        }
        return ServiceResponse.success();

    }
}
